==> ywz_blue_plugin-0.0.5.tar.gz: lib/src/BlueManage.dart <==


part of ywz_blue_plugin;

class BlueManage {

  // 下面的所有UUID及指令请根据实际设备替换
  static const String APP_UUID_SERVICE = "0000ffe0-0000-1000-8000-00805f9b34fb";
  static const String DEVICE_UUID_CHARACTER =
      "0000ffe1-0000-1000-8000-00805f9b34fb";
  static const String APP_UUID_CHARACTER =
      "0000ffe2-0000-1000-8000-00805f9b34fb";
  static const String READ_DEVICE_CONFIGURATION_CHARACTER =
      "0000ffe3-0000-1000-8000-00805f9b34fb";
  static const String ENCRYPT_UUID_CHARACTER =
      "0000ffe4-0000-1000-8000-00805f9b34fb";
  static const String READ_DEVICE_UUID_SERVICE =
      "0000180f-0000-1000-8000-00805f9b34fb";
  static const String READ_DEVICE_UUID_CHARACTER =
      "0000180f-0000-1000-8000-00805f9b34fb";

  static const String READ_DEVICE_CONFIGURATION_UUID_SERVICE =
      "0000180a-0000-1000-8000-00805f9b34fb";
  static const String READ_DEVICE_VERSION_CHARACTER =
      "00002a24-0000-1000-8000-00805f9b34fb";

  // 解除报警
  static const List<int> DIS_WARNING = [33, 32, 17, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2];

  // 呼叫报警器
  static const List<int> SEARCH_DEVICE = [33, 32, 18, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2];

  //
  static const List<int> DIS_BINDING = [33, 32, 19, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2];

  // 报警成功
  static const List<int> WARNING_SUCCESS = [33, 20, 19, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2];

  // 报警失败
  static const List<int> WARING_FAILED = [33, 32, 21, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2];

  // 关机
  static const List<int> POWER_OFF = [33, 32, 22, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -2];

  //开启声音
  static const List<int> VOICE_ON = [33, 67, 7, 2, 50, 50, 3, 8, 50, 25, 0, 1, 50, 25, 5, 9, 50, 8, 25, -2];

  //关闭声音
  static const List<int> VOICE_OFF = [33, 67, 7, 2, 50, 50, 3, 8, 50, 25, 0, 1, 50, 25, 5, 9, 50, 8, 0, -2];

  //单击
  static const int SINGLE = 1;

  //双击
  static const int TWO = 2;

  //长按
  static const int LONG = 17;

  //拔出拉环
  static const int PULLOUT = 33;

  //插入拉环
  static const int PULLIN = 32;

  //没充电
  static const int BATTERY0 = 0;

  //充电中
  static const int BATTERY1 = 1;

  //充满电
  static const int BATTERY2 = 2;

  //低电量
  static const int BATTERY3 = 3;

  BlueManage._();

  static BlueManage _instance = new BlueManage._();

  static BlueManage get instance => _instance;

  static Map<String, BluetoothDevice> _deviceMapConnected = Map();

  static Map<String, BluetoothDevice> _deviceMapUnConnected = Map();

  static Map<String, Map<String, Map<String, BluetoothCharacteristic>>> _map =
      Map();

  //搜索到未连接的Alarm
  static List<BlueDevice> scanDevices = List();

  //搜索到已连接的Alarm
  static List<BlueDevice> conDevices = List();

  BluetoothCharacteristic _getCharacteristic(
      String deviceid, String serviceId, String charId) {
    return _map[deviceid][serviceId][charId];
  }

  Future<bool> checkAlarm(String deviceId) async {
    var c =
    _getCharacteristic(deviceId, APP_UUID_SERVICE, ENCRYPT_UUID_CHARACTER);
    if (c == null) {
      return false;
    } else {
      var randomSixByte = HexUtils.randomSixByte();
      await c.write(randomSixByte);
      var read = await c.read();
      if (HexUtils.isValid(HexUtils.get8Data(randomSixByte), read)) {
        return true;
      } else {
        return false;
      }
    }
  }

  //[0]是电量，[1]是电量状态
  Future<List<int>> getElectricity(String deviceId) async {
    List<int> read = await _getCharacteristic(
            deviceId, READ_DEVICE_UUID_SERVICE, READ_DEVICE_UUID_CHARACTER)
        .read();
    return read;
  }

  //版本信息
  Future<List<int>> getDeviceVersion(String deviceId) async {
    List<int> read = await _getCharacteristic(
            deviceId,
            READ_DEVICE_CONFIGURATION_UUID_SERVICE,
            READ_DEVICE_VERSION_CHARACTER)
        .read();
    return read;
  }

  //硬件配置信息
  Future<List<int>> getDeviceConfiguration(String deviceId) async {
    List<int> read = await _getCharacteristic(
            deviceId, APP_UUID_SERVICE, READ_DEVICE_CONFIGURATION_CHARACTER)
        .read();
    return read;
  }

  //设置硬件监听
  setOnAlarmChange(String deviceId, void onData(int event)) async {
    var characteristic =
    _getCharacteristic(deviceId, APP_UUID_SERVICE, DEVICE_UUID_CHARACTER);
    await characteristic.setNotifyValue(true);
    characteristic.value.listen((value) {
      if (value.length != 0 && value[0] != null) {
        onData(value[0]);
      }
    });
  }

  //设置电池电量监听
  setOnElectricityChange(String deviceId, void onData(List<int> event)) async {
    var characteristic = _getCharacteristic(
        deviceId, READ_DEVICE_UUID_SERVICE, READ_DEVICE_UUID_CHARACTER);
    await characteristic.setNotifyValue(true);
    characteristic.value.listen((value) {
      if (value != null && value.length != 0) {
        onData(value);
      }
    });
  }

  //呼叫报警器
  Future<bool> CallAlarm(String deviceId) async {
    bool write =
    await _getCharacteristic(deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
        .write(SEARCH_DEVICE, withoutResponse: true);
    return write;
  }

  //解除预警
  Future<bool> disWarning(String deviceId) async {
    bool write =
        await _getCharacteristic(deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
            .write(DIS_WARNING, withoutResponse: true);
    return write;
  }

  //设置预警状态
  Future<bool> setWarningSate(String deviceId, bool result) async {
    bool write = await _getCharacteristic(
            deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
        .write(result ? WARNING_SUCCESS : WARING_FAILED, withoutResponse: true);
    return write;
  }

  //报警器关机
  Future<bool> powerOff(String deviceId) async {
    bool write =
        await _getCharacteristic(deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
            .write(POWER_OFF, withoutResponse: true);
    return write;
  }

  //设置硬件声音 open : true 开启 flase 关闭
  Future<bool> setAlarmVoice(String deviceId, bool open) async {
    bool write;
    if (open) {
      write = await _getCharacteristic(
              deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
          .write(VOICE_ON, withoutResponse: true);
    } else {
      write = await _getCharacteristic(
              deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
          .write(VOICE_OFF, withoutResponse: true);
    }
    return write;
  }

  Function(int code) _scanCallback;

  //初始化
  initBlue() {}

  //开始搜索
  startScan(Function(int code) function) {
    _scanCallback = function;
    _initBlue();
  }

  _initBlue() {
    FlutterBlue.instance.isAvailable
        .then((isAvailable) => _isAvailable(isAvailable));
  }

  _isOn(bool isOn) {
    if (isOn) {
      scanDevices.clear();
      _deviceMapUnConnected.clear();
      FlutterBlue.instance
          .scan(timeout: Duration(seconds: 4))
          .listen((scanResult) => _scan(scanResult))
          .onDone(() => _getConnectedDevices());
    } else {
      _scanCallback(Contans.BLUE_OFF);
    }
  }

  _getConnectedDevices() async {
    conDevices.clear();
    _deviceMapConnected.clear();
    var connectedDevices = await FlutterBlue.instance.connectedDevices;
    connectedDevices.forEach((device) {
      if (device.name == "papa01" &&
          !_deviceMapConnected.containsValue(device)) {
        BlueDevice blueDevice = BlueDevice();
        var deviceId = device.id.toString();
        blueDevice.setID(deviceId);
        blueDevice.setName("怕怕Alarm" + deviceId.substring(deviceId.length - 3));
        blueDevice.setConnect(true);
        conDevices.add(blueDevice);
        _deviceMapConnected[device.id.toString()] = device;
      }
    });
    _scanCallback(Contans.SUCCESS);
  }

  _isAvailable(bool isAvailable) {
    if (isAvailable) {
      FlutterBlue.instance.isOn.then((isOn) => _isOn(isOn));
    } else {
      _scanCallback(Contans.UNAVAILABLE);
    }
  }

  _scan(ScanResult data) {
    var device = data.device;
    if (data.device.name == "papa01" &&
        !_deviceMapUnConnected.containsValue(device)) {
      BlueDevice blueDevice = BlueDevice();
      var deviceId = device.id.toString();
      blueDevice.setID(deviceId);
      blueDevice.setName("怕怕Alarm" + deviceId.substring(deviceId.length - 3));
      blueDevice.setConnect(false);
      scanDevices.add(blueDevice);
      _deviceMapUnConnected[deviceId] = device;
    }
  }

  //连接  true 成功 flase 失败
  Future<bool> connect(BlueDevice blueDevice) async {
    BluetoothDevice device = _deviceMapUnConnected[blueDevice.getId()];
    if (device != null) {
      await device.connect();
      if (device.name == "papa01") {
        List<BluetoothService> services = await device.discoverServices();
        services.forEach((service) async {
          var uuid = service.uuid.toString();
          if (uuid == APP_UUID_SERVICE ||
              uuid == READ_DEVICE_UUID_SERVICE ||
              uuid == READ_DEVICE_CONFIGURATION_UUID_SERVICE) {
            service.characteristics.forEach((characteristic) =>
                _dealCharacteristic(
                    device.id.toString(), uuid, characteristic));
          }
        });
        _isOn(true);
        return true;
      } else {
        return false;
      }
    } else {
      return false;
    }
  }

  //断开连接
  disConnect(String deviceId) {
    if(deviceId!=null&&deviceId.isNotEmpty){
      if(_deviceMapConnected.containsKey(deviceId)){
        _deviceMapConnected[deviceId].disconnect();
        _map[deviceId]=null;
      }
    }
  }



  //停止搜索
  stopScan() async {
    await FlutterBlue.instance.stopScan();
  }

  _dealCharacteristic(
      String deviceid, String serviceId, BluetoothCharacteristic cs) async {
    var uuid = cs.uuid.toString();
    if (uuid == DEVICE_UUID_CHARACTER ||
        uuid == APP_UUID_CHARACTER ||
        uuid == READ_DEVICE_CONFIGURATION_CHARACTER ||
        uuid == READ_DEVICE_UUID_CHARACTER ||
        uuid == ENCRYPT_UUID_CHARACTER ||
        uuid == READ_DEVICE_VERSION_CHARACTER) {
      if (_map[deviceid] == null) {
        Map<String, BluetoothCharacteristic> map0 = Map();
        map0[uuid] = cs;
        Map<String, Map<String, BluetoothCharacteristic>> map1 = Map();
        map1[serviceId] = map0;
        _map[deviceid] = map1;
      } else {
        if (_map[deviceid][serviceId] == null) {
          Map<String, BluetoothCharacteristic> map2 = Map();
          map2[uuid] = cs;
          _map[deviceid][serviceId] = map2;
        } else {
          _map[deviceid][serviceId][uuid] = cs;
        }
      }
    }
  }

  getConnectAlarm(String deviceId){

  }
}

==> ywz_blue_plugin-0.0.7.tar.gz: lib/src/BlueManage.dart <==
part of ywz_blue_plugin;

class BlueManage {
  // 下面的所有UUID及指令请根据实际设备替换
  static const String APP_UUID_SERVICE = "0000ffe0-0000-1000-8000-00805f9b34fb";
  static const String DEVICE_UUID_CHARACTER =
      "0000ffe1-0000-1000-8000-00805f9b34fb";
  static const String APP_UUID_CHARACTER =
      "0000ffe2-0000-1000-8000-00805f9b34fb";
  static const String READ_DEVICE_CONFIGURATION_CHARACTER =
      "0000ffe3-0000-1000-8000-00805f9b34fb";
  static const String ENCRYPT_UUID_CHARACTER =
      "0000ffe4-0000-1000-8000-00805f9b34fb";
  static const String READ_DEVICE_UUID_SERVICE =
      "0000180f-0000-1000-8000-00805f9b34fb";
  static const String READ_DEVICE_UUID_CHARACTER =
      "0000180f-0000-1000-8000-00805f9b34fb";

  static const String READ_DEVICE_CONFIGURATION_UUID_SERVICE =
      "0000180a-0000-1000-8000-00805f9b34fb";
  static const String READ_DEVICE_VERSION_CHARACTER =
      "00002a24-0000-1000-8000-00805f9b34fb";

  // 解除报警
  static const List<int> DIS_WARNING = [
    33,
    32,
    17,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    -2
  ];

  // 呼叫报警器
  static const List<int> SEARCH_DEVICE = [
    33,
    32,
    18,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    -2
  ];

  //
  static const List<int> DIS_BINDING = [
    33,
    32,
    19,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    -2
  ];

  // 报警成功
  static const List<int> WARNING_SUCCESS = [
    33,
    20,
    19,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    -2
  ];

  // 报警失败
  static const List<int> WARING_FAILED = [
    33,
    32,
    21,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    -2
  ];

  // 关机
  static const List<int> POWER_OFF = [
    33,
    32,
    22,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    -2
  ];

  //开启声音
  static const List<int> VOICE_ON = [
    33,
    67,
    7,
    2,
    50,
    50,
    3,
    8,
    50,
    25,
    0,
    1,
    50,
    25,
    5,
    9,
    50,
    8,
    25,
    -2
  ];

  //关闭声音
  static const List<int> VOICE_OFF = [
    33,
    67,
    7,
    2,
    50,
    50,
    3,
    8,
    50,
    25,
    0,
    1,
    50,
    25,
    5,
    9,
    50,
    8,
    0,
    -2
  ];

  //单击
  static const int SINGLE = 1;

  //双击
  static const int TWO = 2;

  //长按
  static const int LONG = 17;

  //拔出拉环
  static const int PULLOUT = 33;

  //插入拉环
  static const int PULLIN = 32;

  //没充电
  static const int BATTERY0 = 0;

  //充电中
  static const int BATTERY1 = 1;

  //充满电
  static const int BATTERY2 = 2;

  //低电量
  static const int BATTERY3 = 3;

  BlueManage._();

  static BlueManage _instance = new BlueManage._();

  static BlueManage get instance => _instance;

  static Map<String, BluetoothDevice> _deviceMapConnected = Map();

  static Map<String, BluetoothDevice> _deviceMapUnConnected = Map();

  static Map<String, Map<String, Map<String, BluetoothCharacteristic>>> _map =
      Map();

  //搜索到未连接的Alarm
  static List<BlueDevice> scanDevices = List();

  //搜索到已连接的Alarm
  static List<BlueDevice> conDevices = List();

  BluetoothCharacteristic _getCharacteristic(
      String deviceid, String serviceId, String charId) {
    return _map[deviceid][serviceId][charId];
  }

  Future<bool> checkAlarm(String deviceId) async {
    var c =
        _getCharacteristic(deviceId, APP_UUID_SERVICE, ENCRYPT_UUID_CHARACTER);
    if (c == null) {
      return false;
    } else {
      var randomSixByte = HexUtils.randomSixByte();
      await c.write(randomSixByte);
      var read = await c.read();
      if (HexUtils.isValid(HexUtils.get8Data(randomSixByte), read)) {
        return true;
      } else {
        return false;
      }
    }
  }

  //[0]是电量，[1]是电量状态
  Future<List<int>> getElectricity(String deviceId) async {
    List<int> read = await _getCharacteristic(
            deviceId, READ_DEVICE_UUID_SERVICE, READ_DEVICE_UUID_CHARACTER)
        .read();
    return read;
  }

  //版本信息
  Future<List<int>> getDeviceVersion(String deviceId) async {
    List<int> read = await _getCharacteristic(
            deviceId,
            READ_DEVICE_CONFIGURATION_UUID_SERVICE,
            READ_DEVICE_VERSION_CHARACTER)
        .read();
    return read;
  }

  //硬件配置信息
  Future<List<int>> getDeviceConfiguration(String deviceId) async {
    List<int> read = await _getCharacteristic(
            deviceId, APP_UUID_SERVICE, READ_DEVICE_CONFIGURATION_CHARACTER)
        .read();
    return read;
  }

  //设置硬件监听
  setOnAlarmChange(String deviceId, void onData(int event)) async {
    var characteristic =
        _getCharacteristic(deviceId, APP_UUID_SERVICE, DEVICE_UUID_CHARACTER);
    await characteristic.setNotifyValue(true);
    characteristic.value.listen((value) {
      if (value.length != 0 && value[0] != null) {
        onData(value[0]);
      }
    });
  }

  //设置电池电量监听
  setOnElectricityChange(String deviceId, void onData(List<int> event)) async {
    var characteristic = _getCharacteristic(
        deviceId, READ_DEVICE_UUID_SERVICE, READ_DEVICE_UUID_CHARACTER);
    await characteristic.setNotifyValue(true);
    characteristic.value.listen((value) {
      if (value != null && value.length != 0) {
        onData(value);
      }
    });
  }

  //呼叫报警器
  Future<bool> CallAlarm(String deviceId) async {
    bool write =
        await _getCharacteristic(deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
            .write(SEARCH_DEVICE, withoutResponse: true);
    return write;
  }

  //解除预警
  Future<bool> disWarning(String deviceId) async {
    bool write =
        await _getCharacteristic(deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
            .write(DIS_WARNING, withoutResponse: true);
    return write;
  }

  //设置预警状态
  Future<bool> setWarningSate(String deviceId, bool result) async {
    bool write = await _getCharacteristic(
            deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
        .write(result ? WARNING_SUCCESS : WARING_FAILED, withoutResponse: true);
    return write;
  }

  //报警器关机
  Future<bool> powerOff(String deviceId) async {
    bool write =
        await _getCharacteristic(deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
            .write(POWER_OFF, withoutResponse: true);
    return write;
  }

  //设置硬件声音 open : true 开启 flase 关闭
  Future<bool> setAlarmVoice(String deviceId, bool open) async {
    bool write;
    if (open) {
      write = await _getCharacteristic(
              deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
          .write(VOICE_ON, withoutResponse: true);
    } else {
      write = await _getCharacteristic(
              deviceId, APP_UUID_SERVICE, APP_UUID_CHARACTER)
          .write(VOICE_OFF, withoutResponse: true);
    }
    return write;
  }

  Function(int code) _scanCallback;

  //初始化
  initBlue() {}

  //开始搜索
  startScan(Function(int code) function) {
    _scanCallback = function;
    _initBlue();
  }

  _initBlue() {
    FlutterBlue.instance.isAvailable
        .then((isAvailable) => _isAvailable(isAvailable));
  }

  _isOn(bool isOn) {
    if (isOn) {
      scanDevices.clear();
      _deviceMapUnConnected.clear();
      FlutterBlue.instance
          .scan(timeout: Duration(seconds: 4))
          .listen((scanResult) => _scan(scanResult))
          .onDone(() => _getConnectedDevices());
    } else {
      _scanCallback(Contans.BLUE_OFF);
    }
  }

  _getConnectedDevices() async {
    conDevices.clear();
    _deviceMapConnected.clear();
    var connectedDevices = await FlutterBlue.instance.connectedDevices;
    connectedDevices.forEach((device) {
      if (device.name == "papa01" &&
          !_deviceMapConnected.containsValue(device)) {
        BlueDevice blueDevice = BlueDevice();
        var deviceId = device.id.toString();
        blueDevice.setID(deviceId);
        blueDevice.setName("怕怕Alarm" + deviceId.substring(deviceId.length - 3));
        blueDevice.setConnect(true);
        conDevices.add(blueDevice);
        _deviceMapConnected[device.id.toString()] = device;
      }
    });
    _scanCallback(Contans.SUCCESS);
  }

  _isAvailable(bool isAvailable) {
    if (isAvailable) {
      FlutterBlue.instance.isOn.then((isOn) => _isOn(isOn));
    } else {
      _scanCallback(Contans.UNAVAILABLE);
    }
  }

  _scan(ScanResult data) {
    var device = data.device;
    if (data.device.name == "papa01" &&
        !_deviceMapUnConnected.containsValue(device)) {
      BlueDevice blueDevice = BlueDevice();
      var deviceId = device.id.toString();
      blueDevice.setID(deviceId);
      blueDevice.setName("怕怕Alarm" + deviceId.substring(deviceId.length - 3));
      blueDevice.setConnect(false);
      scanDevices.add(blueDevice);
      _deviceMapUnConnected[deviceId] = device;
    }
  }

  //连接  true 成功 flase 失败
  Future<bool> connect(BlueDevice blueDevice) async {
    BluetoothDevice device = _deviceMapUnConnected[blueDevice.getId()];
    if (device != null && device.name == "papa01") {
      await device.connect();
      List<BluetoothService> services = await device.discoverServices();
      services.forEach((service) async {
        var uuid = service.uuid.toString();
        if (uuid == APP_UUID_SERVICE ||
            uuid == READ_DEVICE_UUID_SERVICE ||
            uuid == READ_DEVICE_CONFIGURATION_UUID_SERVICE) {
          service.characteristics.forEach((characteristic) =>
              _dealCharacteristic(device.id.toString(), uuid, characteristic));
        }
      });
      _deviceMapConnected[blueDevice.getId()] = device;
      _deviceMapUnConnected.remove(blueDevice.getId());
      conDevices.add(blueDevice);
      scanDevices.remove(blueDevice);
      return true;
    } else {
      return false;
    }
  }

  //断开连接
  Future disConnect(String deviceId) async {
    if (deviceId != null && deviceId.isNotEmpty) {
      if (_deviceMapConnected.containsKey(deviceId)) {
       await _deviceMapConnected[deviceId].disconnect();
        _map[deviceId] = null;
      }
    }
  }

  //停止搜索
  stopScan() async {
    await FlutterBlue.instance.stopScan();
  }

  _dealCharacteristic(
      String deviceid, String serviceId, BluetoothCharacteristic cs) async {
    var uuid = cs.uuid.toString();
    if (uuid == DEVICE_UUID_CHARACTER ||
        uuid == APP_UUID_CHARACTER ||
        uuid == READ_DEVICE_CONFIGURATION_CHARACTER ||
        uuid == READ_DEVICE_UUID_CHARACTER ||
        uuid == ENCRYPT_UUID_CHARACTER ||
        uuid == READ_DEVICE_VERSION_CHARACTER) {
      if (_map[deviceid] == null) {
        Map<String, BluetoothCharacteristic> map0 = Map();
        map0[uuid] = cs;
        Map<String, Map<String, BluetoothCharacteristic>> map1 = Map();
        map1[serviceId] = map0;
        _map[deviceid] = map1;
      } else {
        if (_map[deviceid][serviceId] == null) {
          Map<String, BluetoothCharacteristic> map2 = Map();
          map2[uuid] = cs;
          _map[deviceid][serviceId] = map2;
        } else {
          _map[deviceid][serviceId][uuid] = cs;
        }
      }
    }
  }

  getConnectAlarm(String deviceId) {}
}
